/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package chat.business;

import java.util.Vector;
import com.lutris.util.HtmlEncoder;


/**
 *  This class holds all the data about one message (what someone said).
 *  The message is broken down into lines, and each line is HTML encoded.
 *  This makes displaying the message faster. The message is created once,
 *  but displayed over and over to all the listeners.
 */
public class Message {

    /**
     *  Who sent in this bit of text?
     */
    public String name;

    /**
     *  The HTML encoded version of name.
     *  This string is safe to stick into a page.
     */
    public String htmlName;

    /**
     *  The text of the message, in one un-encoded string.
     *  The user may have entered invalid HTML, so use the htmlChunks
     *  when emitting as HTML.
     */
    public String text;

    /**
     *  The text of the message, broken down by lines, then HTML encoded.
     *  These strings are safe to stick into a page.
     */
    public Vector htmlChunks = new Vector();

    /**
     *  The time when this message was added to the discussion.
     *  This is used to delete messages that are too old.
     */
    private long when;

    /**
     *  Create a new message. This does not add it to the discussion.
     *  The HTML fields and time are initialized, though.
     *  @see chat.business.Discussion
     */
    public Message(String name, String text) {
        when = System.currentTimeMillis();
        this.name = name;
        this.htmlName = HtmlEncoder.encode(name);
        this.text = text;
        /*
         *  Break the text into lines, and HTML encode each line.
         */
        if (text.length() == 0)
            htmlChunks.addElement("");
        while (text.length() != 0) {
            int end = text.indexOf("\n");
            if (end != -1) {
              htmlChunks.addElement(HtmlEncoder.encode(text.substring(0,end)));
              text = text.substring(end+1);
            } else {
                htmlChunks.addElement(HtmlEncoder.encode(text));
                text = "";
            }
        }
    }

    
    /**
     *  Get the time when the message was created. The result is in
     *  milliseconds, see System.currentTimeMillis().
     */
    public long getWhen() {
        return when;
    }

}


