/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package phoneList.business;

import java.util.Vector;
import java.sql.SQLException;
import phoneList.data.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.sql.DatabaseManager;
import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.DBQuery;
import com.lutris.appserver.server.sql.DBTransaction;
import com.lutris.appserver.server.sql.ObjectId;

/**
 * PhoneList defines a set of static methods that are used
 * to query and update phone information in the database.
 */
public class PhoneList {

    /**
     * Queries the database for all people/phones.
     *
     * @return Vector[0] contains the people's first names.
     *   Vector[1] contains the people's last names.
     *   Vector[2] contains the phone numbers.
     *   Vector[3] contains the people's identifiers.
     * @exception Exception all exceptions are thrown whenever an
     *   error occurs.
     */
    public static Vector[] getLists() throws Exception {
        Vector[] results = new Vector[4];
        results[0] = new Vector();
        results[1] = new Vector();
        results[2] = new Vector();
        results[3] = new Vector();
        try {
            PersonQuery personQuery = new PersonQuery();
            DBQuery db = Enhydra.getDatabaseManager().createQuery();
            try {
                db.query(personQuery);
                PersonDO personDO = null;
                while ((personDO = (PersonDO) db.next()) != null) {
		    results[0].addElement(personDO.getFirstName());
		    results[1].addElement(personDO.getLastName());
		    results[2].addElement(personDO.getPhoneNumber());
		    results[3].addElement("" + personDO.getOId());
                }
            } finally {
                // You must do this in order to free up
                // any allocated connections and resources.
                db.release();
            }
        } catch (Exception e) {
            // You should deal with the exception.....
            throw e;
        }
        return results;
    }

    /**
     * Creates a new person and adds him/her to the database.
     * Demonstrates how to use a
     * transaction to insert data into the database.
     * 
     * @param firstName the person's first name.
     * @param lastName the person's last name.
     * @param phoneNumber the person's phone number.
     *
     * @exception Exception all exceptions are thrown.  A more
     *   robust application would handle them.....
     */
    public static void addPerson(String firstName,
                                 String lastName,
                                 String phoneNumber)
        throws Exception {
        PersonDO newGuy = new PersonDO();
        newGuy.setFirstName(firstName);
        newGuy.setLastName(lastName);
        newGuy.setPhoneNumber(phoneNumber);
        try {
            DBTransaction db = Enhydra.getDatabaseManager().createTransaction();
            try {
                // Multiple data objects could be update, deleted, inserted
                // here as a single transaction.....
                db.insert(newGuy);
                db.commit();
            } catch (SQLException sqle) {
                db.rollback();
                throw sqle;
            } finally {
                // You must call this to free any resources (such as
                // connections) that were allocated by the transaction.
                db.release();
            }
        } catch (Exception e) {
            // You should deal with any exceptions here....
            throw e;
        }

    }

    /**
     * Deletes a person from the database.  Demonstrates how to use a
     * transaction to delete data in the database.
     * 
     * @param id the person's id
     *
     * @exception Exception all exceptions are thrown.  A more
     *   robust application would handle them.....
     */
    public static void deletePerson(String id) throws Exception {
        PersonDO personDO = getPersonDOById(id);
        if (personDO == null) {
            return;
        }
        try {
            DBTransaction db = Enhydra.getDatabaseManager().createTransaction();
            try {
                // Multiple data objects could be update, deleted, inserted
                // here as a single transaction.....
                db.delete(personDO);
                db.commit();
            } catch (SQLException sqle) {
                db.rollback();
                throw sqle;
            } finally {
                // You must call this to free any resources (such as
                // connections) that were allocated by the transaction.
                db.release();
            }
        } catch (Exception e) {
            // You should deal with any exceptions here....
            throw e;
        }
    }

    /**
     * Modifies a person's profile.  Demonstrates how to use a
     * transaction to update data in the database.
     * 
     * @param id the person's id
     * @param newFirstName the person's new first name.
     * @param newLastName the person's new last name.
     * @param newPhoneNumber the person's new phone number.
     *
     * @exception Exception all exceptions are thrown.  A more
     *   robust application would handle them.....
     */
    public static void modifyPerson(String id,
                                    String newFirstName,
                                    String newLastName,
                                    String newPhoneNumber) 
        throws Exception {
        PersonDO personDO = getPersonDOById(id);
        if (personDO == null) 
            return;
        personDO.setFirstName(newFirstName);
        personDO.setLastName(newLastName);
        personDO.setPhoneNumber(newPhoneNumber);
        try {
            DBTransaction db = Enhydra.getDatabaseManager().createTransaction();
            try {
                // Multiple data objects could be update, deleted, inserted
                // here as a single transaction.....
                db.update(personDO);
                db.commit();
            } catch (SQLException sqle) {
                db.rollback();
                throw sqle;
            } finally {
                // You must call this to free any resources (such as
                // connections) that were allocated by the transaction.
                db.release();
            }
        } catch (Exception e) {
            // You should deal with any exceptions here....
            throw e;
        }
    }

    /**
     * Queries a person from the database.
     *
     * @param id the person's id.
     * @exception Exception all exceptions are thrown.  A more
     *   robust applications would handle them.....
     */
    public static Person getById(String id) throws Exception {
        PersonDO personDO = getPersonDOById(id);
        if (personDO == null) {
            return null;
        }
        return new Person(personDO);
    }
    
    /**
     * Queries a person from the database.
     *
     * @param id the person's id.
     * @exception Exception all exceptions are thrown.  A more
     *   robust applications would handle them.....
     */
    private static PersonDO getPersonDOById(String id) throws Exception {
        ObjectId oid = new ObjectId(id);
        PersonQuery personQuery = new PersonQuery();
        personQuery.setQueryOId(oid);
        DBQuery db = Enhydra.getDatabaseManager().createQuery();
        PersonDO personDO = null;
        try {
            db.query(personQuery);
            personDO = (PersonDO) db.next();
        } finally {
            // Free up any resources.....
            db.release();
        }
        return personDO;
    }
    
} 

