/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

/*-----------------------------------------------------------------------------
 * BasicDO.java
 *   Abstract data object with basic functionality.
 *-----------------------------------------------------------------------------
 */

package phoneList.data;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import java.sql.*;
import java.math.BigDecimal;
import java.util.StringTokenizer;

/**
 * Basic data object.  This could be rolled into CloneableDO and CoreDO.
 * <P>
 * <UL>
 * <LI>Keeps track of whether or not the data object is "dirty"
 * <LI>Keeps track of whether or not the database is being updated for 
 *     this data object
 * <LI>Only updates database if the data object is dirty
 * <LI>Does not update database if another process is doing an update
 * </UL>
 *
 * @author Kristen Pol
 * @since Enhydra2.0
 */
public abstract class BasicDO extends CloneableDO {

    // protected data members

    // Is this data object dirty?
    protected boolean isDirty = false;

    // Is the database entry for this data object being updated?
    protected boolean isExecuting = false;

    // constructors

    // Construct data object from scratch
    public BasicDO()
            throws ObjectIdException, DatabaseManagerException {
	super();
	setOId(Enhydra.getDatabaseManager().allocateObjectId());
	setDirty(true);
    }

    // Construct data object from data in database
    public BasicDO(ResultSet rs)
            throws SQLException, ObjectIdException {
        super(rs);
        setNewVersion(getVersion());
        setDirty(false);
    }

    // public and protected methods

    // Clone this data object uniquely
    public synchronized Object cloneUnique()
	    throws DatabaseManagerException, ObjectIdException {
        BasicDO basicDO = (BasicDO) super.cloneUnique();
        basicDO.setNewVersion(0); 
        basicDO.setDirty(true);
        return basicDO;
    }

    /* ---- SETTER/GETTER METHODS ---- */

    // Set dirty flag
    protected void setDirty(boolean isDirty) {
        this.isDirty = isDirty;
    }

    // Is this data object dirty?
    public boolean isDirty() {
        return this.isDirty;
    }

    // Set executing flag
    protected void setExecuting(boolean isExecuting) {
        this.isExecuting = isExecuting;
    }

    // Is an execute method being called?
    public boolean isExecuting() {
        return this.isExecuting;
    }

    /* ---- INSERT METHODS ---- */

    // Insert the data for this data object in the database
    public PreparedStatement getInsertStatement(DBConnection conn)
            throws SQLException {
        String sql = "insert into " + getTableName() + " \n" +
            "values " + createQuestionMarks(2 + getNumParams());
        PreparedStatement stmt = conn.prepareStatement(sql);
        int param = 1;
        stmt.setBigDecimal(param++, getOId().toBigDecimal());
        stmt.setInt(param++, getVersion());
	param = setParameters(stmt, param);
        return stmt;
    }

    // Insert data into database
    public synchronized void executeInsert(DBConnection conn) 
            throws SQLException {
        if (! isExecuting()) {
            super.executeInsert(conn);
            setExecuting(true);
        }
    }

    // What to do after insert is done
    public void finalizeInsert(boolean success) {
        boolean p = persistent;
        super.finalizeInsert(success);
        if (p == true) {
            persistent = true; // Bug fix for CoreDO! (it overwrites persistent)
        }
	done(success);
    }

    /* ---- UPDATE METHODS ---- */

    // Update the data for this data object in the database
    public PreparedStatement getUpdateStatement(DBConnection conn)
            throws SQLException {
        String sql =
            "update " + getTableName() + " \n" +
            "set " +
            "version = ?, " +
	    createUpdateSql() +
            " where oId = ? and version = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        int param = 1;
        stmt.setInt(param++, getNewVersion());
	param = setParameters(stmt, param);
        stmt.setBigDecimal(param++, getOId().toBigDecimal());
        stmt.setInt(param++, getVersion());

        return stmt;
    }

    // Update data in database
    public synchronized void executeUpdate(DBConnection conn) 
            throws SQLException {
        if (isDirty() && ! isExecuting()) {
            super.executeUpdate(conn);
            setExecuting(true);
        }
    }

    // What to do after update is done
    public void finalizeUpdate(boolean success) {
        super.finalizeUpdate(success);
	done(success);
    }

    /* ---- DELETE METHODS ---- */

    // Delete the data for this data object from the database
    public PreparedStatement getDeleteStatement(DBConnection conn)
            throws SQLException {
        String sql = "delete from " + getTableName() + " where oId = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        return stmt;
    }

    // Delete data from database
    public void executeDelete(DBConnection conn)
            throws SQLException {
        if (isPersistent()) { 
	    while (! isExecuting()) {
		super.executeDelete(conn);
		setExecuting(true);
	    }
        }
    }

    // What to do after delete is done
    public void finalizeDelete(boolean success) {
        super.finalizeDelete(success);
	done(success);
    }

    /* ---- HELPER METHODS ---- */

    // What is the name of the database table for this data object?
    protected abstract String getTableName();

    // Sql string to use for updating the database
    protected abstract String[] getParameters();

    // Set data for the given prepared statement
    protected abstract int setParameters(PreparedStatement stmt, int param)
	throws SQLException;

    // Sql string to use for updating the database
    protected String createUpdateSql() {
	String[] parameters = getParameters();
	String sql = "";
	if (parameters != null) {
	    int length = parameters.length;
	    for (int i = 0; i < length; i++) {
		sql += parameters[i] + "= ?";
		if (i < (length - 1)) {
		    sql += ",";
		}
	    }
	}
	return sql;
    }

    // Creates string like "(?,?,?)" with numberMarks number of question marks
    protected String createQuestionMarks(int numberMarks)
            throws SQLException {
	String sql = "(";
	for (int i = 0; i < numberMarks; i++) {
	    sql += "?";
	    if (i < (numberMarks - 1)) {
		sql += ",";
	    }
	}
	sql += ")";
	return sql;
    }

    // How many parameters does this data object have?
    protected int getNumParams() {
	String updateSql = createUpdateSql();
	StringTokenizer tokenizer = new StringTokenizer(updateSql, "?");
	return tokenizer.countTokens();
    }

    // Set the string in the statement
    protected int setString(PreparedStatement stmt, int param, String data) 
	    throws SQLException {
	if (data != null) {
	    stmt.setString(param++, data);
	} else {
	    stmt.setNull(param++, Types.VARCHAR);
	}
	return param;
    }

    // private methods

    private void done(boolean success) {
	setDirty(! success);
	setExecuting(false);
    }
}
